# mDash library API reference

This howto documents mDash library API. The library could be used
in Arduino IDE or ESP-IDF environments, and it is located at
https://github.com/cesanta/mDash.

<div class="toclist"></div>

## Initialisation

### mDashBegin: initialise mDash library

```c
void mDashBegin(DEVICE_PASSWORD);
```
This function should be the first mDash function called in the  `setup()` .
It initialised the mDash library. If a device has WiFi and cloud credentials
setup (via the Serial provisioning process, or via the API), then
The library creates TLS connection to the
[mdash.net](http://mdash.net/) service and keeps it alive, reconnecting
automatically when required. `DEVICE_PASSWORD` is an mDash device password.
Example usage for Arduino IDE:

```c
#define MDASH_APP_NAME "MinimalApp"
#include <mDash.h>

#include <WiFi.h>

#define WIFI_NETWORK "MyWifiNetworkName"
#define WIFI_PASSWORD "MyWifiPassword"
#define DEVICE_PASSWORD "mDashDeviceToken"

void setup() {
  Serial.begin(115200);
  WiFi.begin(WIFI_NETWORK, WIFI_PASSWORD);
  mDashBegin(DEVICE_PASSWORD);
}

void loop() {
  delay(100);
}
```


## Configuration


### mDashConfigGet: get configuration parameter
```c
int mDashConfigGet(const char *name, char *buf, size_t bufsize);
```

Get configuration parameter `name` stored in the `mdash.cfg` file.

Return value:
- `0`: the config variable `name` was found and copied into `buf`
- `-3`: the config variable `name` was not found in `mdash.cfg`
- `-2`: the destination buffer size `bufsize` is too small to hold the value
- `-1`: the cofig file `mdash.cfg` is absent or could not be opened

### mDashConfigSet: set configuration parameter
```c
int mDashConfigSet(const char *name, const char *value);
```

Set the config variable `name` to `value` by saving it to the config
file `mdash.cfg`. If `value` is `NULL` or an empty string, the configuration
variable is removed from the config file.

Return value:
- `0`: successfully saved
- `-1`: error opening config file

Some of the configuration variables are used by the library:
- `wifi.sta.ssid`: WiFi network name
- `wifi.sta.pass`: WiFi password
- `device.pass`: registered device password (token)
- `device.public_key`: access token for the external customers
- `debug.level`: logging level. Default is 2. Use 0 to disable, 3 to verbose

### mDashCLI: proccess command input

```c
void mDashCLI(unsigned char input_byte);
```

Process one input byte (e.g. from Serial or from network) of the configuration
command line interface. Commands are separated by a new line.
Currently supported commands are:

- `set NAME VALUE` - set config variable. Use `-` value to remove variable
- `ll LEVEL` - set log level, `LEVEL` is a number from 0 (no log) to 3 (debug)
- `reboot` - reboot device

> The typical usage is to provision device to WiFi and/or cloud. Example:
> ```c
>   // Until connected to the cloud, enable provisioning over serial
>   while (mDashGetState() != MDASH_CONNECTED)
>     if (Serial.available() > 0) mDashCLI(Serial.read());
> ```

## RPC - handle remote commands

- https://github.com/cesanta/mjson - JSON-RPC API reference.
- https://github.com/cesanta/mDash/blob/master/examples/GpioReadWrite/GpioReadWrite.ino - Arduino example

## Reporting data

### mDashShadowUpdate: update device shadow
```c
int mDashShadowUpdate(const char *json_format, ...);
```

Update device shadow.

> Example: report free RAM to the shadow
> ```c
>   mDashShadowUpdate("{%Q:{%Q:{%Q:%d}}}",
>                     "state", "reported", "RAM", mDashGetFreeRam());
> ```

Return value: 1 if the update was sent, 0
if not (e.g. because a network is down).


### mDashNotify: send arbitrary notification
```c
int mDashNotify(const char *name, const char *json_format, ...);
```

Send notification `name` to mDash. Return value: 1 if the update was sent, 0
if not (e.g. because a network is down).

Notifications are supposed to be caught by the user's notification catcher.
If they are not, they are simply dropped on the floor and have no effect.

However, certain notifications are handled by mDash automatically. This is a list:

- `Shadow.Update`, params: `{"state":{...}}` - sends device shadow update.
  The `mDashShadowUpdate` is just a wrapper for this notification
- `Shadow.Get`, params: none. This notification makes mDash to call a
  `Shadow.Get` RPC function on a device, passing a complete device shadow
- `HTTP.Request`, params: `{"url":"..","method":"..","body":".."}` - call
  some remote HTTP service with given parameters
- `DB.Store`, params: `["query1", ...]` - calls database SQL query with
  parameters. Used to store device data. See `mDashStore()` section below
- `DB.NewData`, params: none. This notification is used by the `mDashStore()`
  function to notify mDash that a new data is available on device's flash


### mDashStore: store data into the database
```c
void mDashStore(const char *name, const char json_format *, ...);
```

Store data into a database. `name` is the query name in the `data_write`
device ACL, and `json_format` specify query parameters. Query parameters
may be string or numeric. The following special tokens are automatically
replaced by mDash before storing into the database:

- `DEVICE_ID` is substituted with an actual device ID that makes a call
- `CUSTOMER_ID` is substituted with an actual customer ID a calling device belongs to

> Note: this call does not send data to the mDash directly. Instead, it stores
> data into a local file `mdash.unsent` file, and raises a notification event
> `DB.NewData` to the mDash. When mDash receives that notification (or, when
> a device gets connected), mDash fetches `mdash.unsent` file from a device,
> saves all records to a database, and deletes `mdash.unsent` on a device.
> This pipelining logic protects data from the occasional network outages.
> If `mdash.unsent` file size gets over the 32k limit,
> `mdash.unsent` file is renamed to `mdash.unsent.old` which is processed
> my mDash too. Such rotation logic allows to keep up to 64k of the most recent data.


Example:

```c
double temperature = 123.45;
mDashStore("query1", "[%Q, %Q, %g]", "2019-12-31 23:59:59", "DEVICE_ID", temperature);
```

The associated device ACL must have a `query1` key in the `data_write` section,
and have query parameters that match the `mDashStore()` call:

```javascript
{
  "public_key": "...",
  "data_write": {
    "query1": "INSERT INTO tab1 (timestamp,device_id,temperature) VALUES(?,?,?)"
  },
  ...
}
```

> Note: if you want to bypass storing data on flash, and send data immediately,
> call `mDashNotify("DB.Store", ...)`, passing query name with parameters.
> In this case, data could be lost on network failures, but would not hit
> flash. Example:
```c
mDashNotify("DB.Store", "[%Q,%Q,%Q,%g]", "query1", "2019-12-31 23:59:59", "DEVICE_ID", temperature)`.
```


## Events


### mDashRegisterEventHandler: register an event handler
```c
enum {
  // Events defined by the mDash library
  MDASH_EVENT_NETWORK_LOST = 0,       // event_data: NULL
  MDASH_EVENT_NETWORK_AP = 1,         // event_data: NULL
  MDASH_EVENT_NETWORK_CONNECTED = 2,  // event_data: NULL
  MDASH_EVENT_CLOUD_CONNECTED = 3,    // event_data: NULL

  // Events defined by user
  MDASH_EVENT_USER = 100,           // Starting number for user-based events
};
typedef void (*evh_t)(void *event_data, void *userdata);
void mDashRegisterEventHandler(int event, evh_t fn, void *userdata);
```

Register an event handler. Events are used for the asynchronous notification.
Some events are triggered by the mDash library. User code can also trigger
custom events - for that, event numbers higher than `MDASH_EVENT_USER` must
be used.

Each event has an associated `void *` pointer which carries an event-specific
data, or `NULL`.


### mDashTriggerEvent: trigger an event

```c
int mDashTriggerEvent(int event, void *event_data);
```

Trigger an event. This function calls all registered event handlers
synchronously, passing `event_data` to them.

Return a number of invoked handlers.



## Utility

### mDashGetState: get current device state
```c
int mDashGetState(void);
```

Return current device connection state, one of `MDASH_EVENT_*` values.


### mDashGetFreeRam: get free RAM
```c
unsigned long mDashGetFreeRam(void);

```
Return free RAM in bytes


### mDashGetSdkVersion: get ESP-IDF version

```c
const char *mDashGetSdkVersion(void);
```

Return ESP-IDF version used to build mDash library.


### mDashSetLogLevel: set serial debug level

```c
void mDashSetLogLevel(int logLevel);
```

Set serial debug level. The default is 2. Acceptable levels are:
- `0` : no debug logs
- `1` : only critical messages
- `2` : critical and info messages (default)
- `3` : critical, info and debug messages
